package com.changhong.epc.count.service.count.data.paramiter;

import com.changhong.epc.bean.tenant.bill.BillData;
import com.changhong.epc.bean.tenant.bill.IOCRBill;
import com.changhong.epc.bean.tenant.bill.RuleMsg;
import com.changhong.epc.bean.tenant.system.ErrSystemMsg;
import com.changhong.epc.count.service.count.model.CountParamMax;

import java.util.List;
import java.util.Map;
import java.util.Set;

/**
 * 参数迭代器 （如行程）
 * @ClassName: IParamIter
 * @author [九鼎联合科技]
 * @date 2017年8月30日 下午9:39:01
 * @param <T> 参数
 */
public interface IParamIter<T> {

	/**
	 * set数据源 
	 * unitData 所有行程
	 * billList 所有票据
	 * @param param
	 * @param billList
	 */
	IParamIter<T> setDataSource(CountParamMax param, List<BillData> billList);

	/**
	 * 移动坐标到下一个位置
	 * @Title: next
	 * @param @return    设定文件
	 * @return T    返回类型
	 * @throws
	 */
	T next();
	
	/**
	 * 判断是否还有下一个位置
	 * @Title: hasNext
	 * @param @return    设定文件
	 * @return Boolean    返回类型
	 * @throws
	 */
	Boolean hasNext();

	/**
	 * 获取当前对象 
	 * @Title: current
	 * @param @return    设定文件
	 * @return T    返回类型
	 * @throws
	 */
	T current();
	
	
	/**
	 * 获取所有未关联行程的票据 信息
	 * @return
	 */
	Set<IOCRBill> getNoJoinAllBillInfo();
	
	/**
	 * 获得所有为管理行程的总金额
	 * @return
	 */
	Double getNoJoinAllBillTotalMoney();
	
	/**
	 * 获得原始所有行程
	 * @return
	 */
	List<T> getOldUnitInfos();
	
	/**
	 * 获得当前行程 原始数据
	 * @return
	 */
	T getEachOldUnitInfo();

	/**
	 * 获得票据信息
	 * @return
	 */
	Map<Integer, List<IOCRBill>> getBillList();

	/**
	 * 获得规则检查信息
	 * @return
	 */
	List<RuleMsg> getRuleMsg();

	/**
	 * 获得错误信息
	 * @return
	 */
	List<ErrSystemMsg> getErrSystemMsgs();

	List<IOCRBill> getAllBillList();

	String getBillType();
}
