package com.changhong.epc.count.service.count.data.paramiter.impl;

import com.changhong.epc.bean.count.MasterInfo;
import com.changhong.epc.bean.tenant.bill.BillData;
import com.changhong.epc.bean.tenant.bill.IOCRBill;
import com.changhong.epc.bean.tenant.bill.OCRBillHeaderEntity;
import com.changhong.epc.bean.tenant.bill.RuleMsg;
import com.changhong.epc.bean.tenant.master.MasterEleValue;
import com.changhong.epc.bean.tenant.system.ErrSystemMsg;
import com.changhong.epc.config.tool.ContextEnvironment;
import com.changhong.epc.count.service.count.data.paramiter.IParamIter;
import com.changhong.epc.count.service.count.data.tool.ICorrelMaster;
import com.changhong.epc.count.service.count.model.CountParamMax;
import com.changhong.epc.count.service.count.model.UnitInfo;
import com.changhong.epc.count.service.tool.MasterListTool;
import com.changhong.epc.rely.api.tool.StringUtils;
import com.iemsoft.framework.cloud.core.tools.JSONTool;
import com.iemsoft.framework.cloud.core.tools.ObjectUtil;
import lombok.Getter;
import lombok.Setter;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections4.CollectionUtils;
import org.springframework.context.annotation.Primary;
import org.springframework.context.annotation.Scope;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.*;
import java.util.stream.Collectors;

import static com.changhong.epc.constter.system.SystemConst.PRIVATE_TRAVEL;
import static com.changhong.epc.constter.system.SystemConst.PUBLIC_TRAVEL;

/**
 * 行程 票据 迭代器
 * @author wangZX
 *
 */
@Slf4j
@Service("paramIter")
@Scope("prototype")
@Primary
public class ParamIter implements IParamIter<UnitInfo> {

	private List<UnitInfo> units;

	@Setter
	private Integer index = -1;

	private String type;

	/**
	 * 错误信息
	 */
	@Getter
	private List<ErrSystemMsg> errSystemMsgs = new ArrayList<>();
	
	/**
	 * 查找相关联的主数据
	 */
	@Resource
	private ICorrelMaster<UnitInfo> correlMaster;

	/**
	 * 无形成票据
	 */
	private Set<IOCRBill> noJoinBill;

	private List<OCRBillHeaderEntity> billList = new ArrayList<>();
	
	/**
	 * 初始化 将票据 铺开给行程
	 * @param units
	 */
	public void init(List<UnitInfo> units) {
		this.units = new ArrayList<>();
		
		for (UnitInfo unitInfo : units) {
			unitInfo.setUuid(UUID.randomUUID().toString());
//			boolean flog = true;
//			if(unitInfo.getBillInfos() != null){
//				for(OCRBillHeaderEntity header : unitInfo.getBillInfos()){
//					boolean flogBody = true;
//					for(OCRBillBodyEntity body : header.getBillBodyList()){
//						this.units.add(new UnitInfo(unitInfo, body, new HashSet<>()));
//						flogBody = false;
//						flog = false;
//					}
//					if(flogBody){
//						this.units.add(new UnitInfo(unitInfo, header, new HashSet<>()));
//						flog = false;
//					}
//				}
//			}
//			if(flog)
//			unitInfo.setBillInfo(new ArrayList<>(unitInfo.getBillInfos()));
			unitInfo.setBillInfo(marge(unitInfo.getBillInfo()));
			this.units.add(unitInfo);
		}

		setAllJoinMaster();
	}

	public List<IOCRBill> marge(List<? extends IOCRBill> iocrBills){
		iocrBills =
				iocrBills.stream()
						.sorted((a, b)->
								Optional.ofNullable(a)
										.map(IOCRBill::getCostType)
										.map(IOCRBill.CostType::getCode)
										.orElse("")
									.compareTo(
											Optional.ofNullable(b)
													.map(IOCRBill::getCostType)
													.map(IOCRBill.CostType::getCode)
													.orElse("")
									)
						)
						.collect(Collectors.toList());

		Set<IOCRBill> res = new HashSet<>();

		IOCRBill last = null;

		for(IOCRBill bill : iocrBills){
			if(last == null){
				last = bill;
				continue;
			}
			if(last.getBillByType(
					Optional.ofNullable(bill)
							.map(IOCRBill::getCostType)
							.map(IOCRBill.CostType::getCode)
							.orElse(""))
					){
				last.appendBill(bill);
			}else{
				res.add(last);
				last = bill;
			}
		}
		res.add(last);
		return new ArrayList<>(res);
	}

	public Set<MasterEleValue> getMasterInfos(){
		Set<MasterEleValue> arrays = new LinkedHashSet<>();
		for (UnitInfo unitInfo : this.units) {
			MasterListTool.add(
					Boolean.TRUE
					, arrays
					, ml->Objects.equals("From_site", ml.getAlias()) || Objects.equals("From_site_land", ml.getAlias())
					, unitInfo.getUnitInfo().stream()
							.filter(this::isEmpty)
							.map(masterInfo ->new MasterEleValue(masterInfo.getMdCode(), getVal(masterInfo.getMeVal()))).toArray(MasterEleValue[]::new)
			);
		}
		return arrays;
	}

	protected void setAllJoinMaster(){
		Map<MasterEleValue, MasterEleValue> joinMasterInfos = correlMaster.getJoinMasterInfos(getMasterInfos());
		this.units.stream().forEach(
				unitInfo -> unitInfo
						.getUnitInfo()
						.stream()
//						.filter(this::isEmpty)
				.forEach(masterInfo -> {
					MasterEleValue joinInfo = joinMasterInfos.get(new MasterEleValue(masterInfo.getMdCode(), getVal(masterInfo.getMeVal())));
					if(joinInfo != null && ObjectUtil.notEmpty(joinInfo.getJoinDefines())){
						// 添加关联主数据
						unitInfo.addMdJoinAll(Boolean.FALSE, joinInfo.getJoinDefines().stream()
								.map(masterEleValue -> masterEleValue.toMasterInfo()).toArray(MasterInfo[]::new));
					}
					masterInfo.setMeVal(getVal(masterInfo.getMeVal()));
					unitInfo.addMdJoinAll(Boolean.TRUE, masterInfo);
				})
		);
	}

	protected String getVal(Object val){
		if(val instanceof List){
			return ObjectUtil.empty(val) ? "" : ((List<String>) val).get(0);
		}else{
			return val == null ? "" : val.toString();
		}
	}

	protected boolean isEmpty(MasterInfo masterInfo){
		return StringUtils.isNotBlank(Objects.toString(masterInfo.getMeVal())) && StringUtils.isNotBlank(masterInfo.getMdCode());
	}

	@Override
	public UnitInfo next() {
		return units.get(++index);
	}
	
	@Override
	public Boolean hasNext() {		
		return units.size() > index+1;
	}

	@Override
	public UnitInfo current() {
		return units.get(index);
	}

	@Override
	public ParamIter setDataSource(CountParamMax param, List<BillData> billList) {
		this.type = new ContextEnvironment().travelType(
				param.getCtId()
				, ()->PUBLIC_TRAVEL
				, ()->PRIVATE_TRAVEL
				, ()->null);
		List<UnitInfo> unitData = param.getUnitData();
		noJoinBill = new HashSet<>();
		if(billList != null){
			for (UnitInfo unit : unitData) {
				findGroup(unit,billList);
			}
			// 解决没有行程无票据问题
			if(ObjectUtil.empty(unitData)){
				findGroup(null, billList);
			}
		}
		init(unitData);
		log.warn("整理后的参数集合:{}", JSONTool.toJson(this.units));
		index = -1;
		log.debug("票据数量:{}", CollectionUtils.size(billList));
		log.debug("票据内容:{}", JSONTool.toJson(billList));
		return this;
	}


	/**
	 * 对票据进行分组
	 * @param unit
	 * @param billList
	 */
	private void findGroup(UnitInfo unit, List<BillData> billList) {
		for (BillData billData : billList) {
			// 找到行程对应的票据  进行赋值
			OCRBillHeaderEntity billHead;
			try {
				billHead = JSONTool.toObj(billData.getBillInfo(), OCRBillHeaderEntity.class);
			} catch (Exception e) {
				log.error(e.getMessage());
				billHead = null;
			}
			// 票据规则检查为忽略状态
			if(billHead == null || billHead.isIgnore()){
				continue;
			}
			this.billList.add(billHead);

			if(unit != null) {
				if (this.type == null || Objects.equals(billData.getJId(), unit.getUnitId())) {
					unit.addBillInfo(getIOCRBill(billHead));
				}
			}
			// 添加无行程票据信息
			if(billData.getJId() == null){
				noJoinBill.addAll(getIOCRBill(billHead));
//				noJoinBill.add(billHead);
			}
		}
	}

	public List<IOCRBill> getIOCRBill(OCRBillHeaderEntity billHead){
		if(ObjectUtil.empty(billHead.getBillBodyEntityList()))
			return Arrays.asList(billHead);
		return Arrays.asList(billHead.getBillBodyEntityList().stream().toArray(IOCRBill[]::new));
	}

	@Override
	public Set<IOCRBill> getNoJoinAllBillInfo() {
		return noJoinBill;
	}


	/**
	 * 获取无关联行程票据金额
	 */
	@Override
	public Double getNoJoinAllBillTotalMoney() {
		Double sum = 0D;
		for (IOCRBill iocrBill : noJoinBill) {
			sum += iocrBill.getBillMoney();
		}
		log.debug("无行程票据金额：{}，无行程票据：{}", sum, JSONTool.toJson(noJoinBill));
		return sum;
	}

	/**
	 * 获得原始所有行程
	 */
	@Override
	public List<UnitInfo> getOldUnitInfos() {
		return units;
	}

	/**
	 * 获得当前行程 原始数据
	 */
	@Override
	public UnitInfo getEachOldUnitInfo() {
		return this.current();
	}

	@Override
	public Map<Integer, List<IOCRBill>> getBillList() {
		if(ObjectUtil.empty(this.units)){
			return new HashMap<>();
		}
		Map<Integer, List<IOCRBill>> billList = new HashMap<>();
		this.units.stream()
				.filter(unitInfo ->
						ObjectUtil.notEmpty(unitInfo.getUuid())
					&& ObjectUtil.notEmpty(unitInfo.getBillInfo())
				)
				.forEach(unitInfo -> billList.put(unitInfo.getUnitId(), unitInfo.getBillInfo()));
		return billList;
	}

	@Override
	public List<RuleMsg> getRuleMsg() {
		List<RuleMsg> ruleMsgs = new ArrayList<>();
		this.billList.stream().forEach(bill->ruleMsgs.addAll(bill.getRuleMsg()));
		return ruleMsgs;
	}

	@Override
	public List<IOCRBill> getAllBillList() {
		if(ObjectUtil.empty(this.billList)){
			return new ArrayList<>();
		}
		List<IOCRBill> allBillLIst = new ArrayList<>();
		billList.stream().forEach(bill->{
			allBillLIst.addAll(getIOCRBill(bill));
		});
		return allBillLIst;
	}

	@Override
	public String getBillType() {
		return this.type;
	}

}
