package com.changhong.epc.count.service.count.impl;

import com.changhong.epc.bean.tenant.bill.BillData;
import com.changhong.epc.bean.tenant.norm.extend.AppStanderExtend;
import com.changhong.epc.bean.tenant.norm.extend.SuppStanderExtend;
import com.changhong.epc.config.tool.ContextEnvironment;
import com.changhong.epc.constter.system.SystemClients;
import com.changhong.epc.count.service.count.IExpendCostCount;
import com.changhong.epc.count.service.count.data.condition.impl.CountConditionIterImpl;
import com.changhong.epc.count.service.count.data.paramiter.impl.ParamIter;
import com.changhong.epc.count.service.count.model.CountParamMax;
import com.changhong.epc.count.service.count.model.CountResultLast;
import com.changhong.epc.count.service.count.model.UnitInfo;
import com.changhong.epc.count.service.count.performer.ICountPerformer;
import com.changhong.epc.count.service.count.performer.impl.CountPerformer;
import com.changhong.epc.rely.api.epc.tenant.BillInfoApi;
import com.changhong.epc.rely.api.epc.tenant.SuppStanderApi;
import com.iemsoft.framework.cloud.core.tools.ObjectUtil;
import com.iemsoft.framework.cloud.core.tools.SpringUtil;
import org.apache.commons.lang3.math.NumberUtils;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.Collections;
import java.util.List;

/**
 * 补助标准
 * @author wangZX
 *
 */
@Service("expendCostCount")
public class ExpendCostCountImpl implements IExpendCostCount, SystemClients {

	@Resource
	private SuppStanderApi suppStanderApi;

	@Resource
	private BillInfoApi billInfoApi;

	/**
	 * 计算费用
	 */
	@Override
	public CountResultLast costCount(CountParamMax param){
		ContextEnvironment context = new ContextEnvironment();
		/**
		 * 1.获取补助标准
		 */
		List<SuppStanderExtend> suppStanderExtends = suppStanderApi.getSuppStanderExtend(NumberUtils.createInteger(param.getCtId()));

		/**
		 * 2.获取行程票据信息
		 */
		List<BillData> billList;
		if(ObjectUtil.empty(param.getOrderCode())){
			billList = Collections.EMPTY_LIST;
		}else {
			billList = billInfoApi.getBillData(param.getOrderCode());
		}
		/**
		 * 4.获取计算费用执行器
		 */
		ICountPerformer<CountResultLast, AppStanderExtend, SuppStanderExtend, UnitInfo> countPerformer
				= context.travelType(
						param.getCtId()
						// 国外
						, ()->SpringUtil.getBean("publicCorrencyCountPerformer", CountPerformer.class)
						// 国内
						, ()->SpringUtil.getBean("countPerformer", CountPerformer.class)
						// 其它
						, ()->SpringUtil.getBean("countPerformer", CountPerformer.class)
				);

		context.travelType(
				param.getCtId()
				// 国外
				, ()->{
					countPerformer.setIParamIter(
							SpringUtil.getBean(ParamIter.class)
									.setDataSource(param, billList));
					return true;
				}
				// 国内
				, ()->{
					countPerformer.setIParamIter(
							SpringUtil.getBean(ParamIter.class)
									.setDataSource(param, billList));
					return true;
				}
				// 其它
				, ()->{
					countPerformer.setIParamIter(
							SpringUtil.getBean("otherTypeParamIter", ParamIter.class)
									.setDataSource(param, billList));
					return true;
				}

		);

		/**
		 * 添加 计算费用计算器 依赖参数
		 */
		countPerformer.setICountConditionIter(
				SpringUtil.getBean(CountConditionIterImpl.class).setDataSource(suppStanderExtends));
//		countPerformer.setIParamIter(
//				SpringUtil.getBean(ParamIter.class).setDataSource(param, billList));

		/**
		 * 执行费用计算
		 */
		return countPerformer.execute();
	}
}
