package com.changhong.epc.count.service.count.logic.impl;

import com.changhong.epc.bean.count.msg.AppStanderMsg;
import com.changhong.epc.bean.tenant.norm.way.Way;
import com.changhong.epc.count.service.count.logic.ILastMoneyCount;
import com.changhong.epc.count.service.count.model.*;
import com.iemsoft.framework.cloud.core.tools.DateTool;
import com.iemsoft.framework.cloud.core.tools.ObjectUtil;
import org.springframework.stereotype.Service;

import java.util.*;
import java.util.Map.Entry;


/**
 * 对补助标准结果处理计算   
 * 提供聚合函数  如  最大   最小  最先到达  最后到达    平均  
 * @ClassName: LastMoneyCount
 * @author [九鼎联合科技]
 * @date 2017年7月17日 上午11:31:25
 */
@Service("lastMoneyCount")
public class LastMoneyCountImpl implements ILastMoneyCount{
	

	
	/**
	 * 根据时间进行分组
	 */
	private Map<GroupKey, List<CountResult>> dateGroupIndex = new HashMap<>();
	private List<List<CountResult>> groups = new ArrayList<>();
	
	
	/**
	 * 00001
	 * 分组方法 记录每一组下标
	 * @Title: grouping
	 * @param
	 * @return void    返回类型
	 * @throws
	 */
	protected void grouping(List<CountResultExtend> resultMoney){
		
		
		for(int i=0; i < resultMoney.size(); i++){
			getFormDay(resultMoney, i);	
		}
		adjustGroup();

	}
	

	/**
	 * 调整分组内数据  剔除 单条 及以下 时间行程
	 * @Title: adjustGroup
	 * @param
	 * @return void    返回类型
	 * @throws
	 */
	private void adjustGroup() {
		List<GroupKey> removes = new ArrayList<>();
		for (Entry<GroupKey, List<CountResult>> entry : dateGroupIndex.entrySet()) {
			if(entry.getValue().size() <= 1){
				removes.add(entry.getKey());
			}
		}
		for (GroupKey rm : removes) {
			dateGroupIndex.remove(rm);
		}
		
		
		for (Entry<GroupKey, List<CountResult>> entry : dateGroupIndex.entrySet()) {
			groups.add(entry.getValue());
		}
		
	}

	private void addGroup(GroupKey formDay, CountResult CountResult) {
		List<CountResult> group = dateGroupIndex.get(formDay);
		if(group != null){
			group.add(CountResult);
		}else{
			List<CountResult> arrayList = new ArrayList<>();
			arrayList.add(CountResult);
			dateGroupIndex.put(formDay, arrayList);
		}
	}
	
	/**
	 * 获得日期 key
	 * @Title: getFormDay
	 * @param @param resultMoney
	 * @param @param i
	 * @param @return    设定文件
	 * @return String    返回类型
	 * @throws
	 */
	private void getFormDay(List<CountResultExtend>resultMoney, Integer i){


		List<CountResult> money = resultMoney.get(i).getCountResults();
		if(ObjectUtil.empty(money)){
			return;
		}
		for (CountResult countResult : money) {
			if(ObjectUtil.notEmpty(countResult.getFormDay())){
				GroupKey groupKey = new GroupKey(DateTool.dateToString(DateTool.stringToDate(countResult.getFormDay()), DateTool.DATE_PATTERN),countResult.getSsId());
				addGroup(groupKey,countResult);
			}
		}
	}
	
	
	public List<CountResult> find(AmountKey ak){
		for (List<CountResult> list : groups) {
			CountResult mm = list.get(0);
			if(Objects.equals(ak.fylx, mm.getJpCdCode()) 
					&& Objects.equals(ak.bxfs, mm.getWay())
					&& Objects.equals(ak.id, mm.getSsId())
					){
				return list;
			}
		}
		
		return null;
	}
	
	@Override
	public List<CountResultExtend> count(Map<AmountKey, String> amountLastCount,
												List<CountResultExtend> resultMoney) {
		
		grouping(resultMoney);
		
		for (Map.Entry<AmountKey, String> entry: amountLastCount.entrySet()) {
			// 1.识别 聚合函数类型
			// System.err.println("=============================="+entry.getValue()+"===========================");
			
			List<CountResult> fin = find(entry.getKey());
			if(fin == null)
				continue;
			switch (entry.getValue()) {
			
			// 第一个
			case LM_FIRST:
				first(fin);
				break;
			// 最后一个 1
			case LM_LAST:
				last(fin);	
				break;
			// 最大
			case LM_MAX:
				max(fin);
				break;
			// 最小 1
			case LM_MIN:
				min(fin);
				break;
			// 平均 1
			case LM_AVG:
				avg(fin);
				break;
			default:
				break;
			}
		}
		
		filterResult(resultMoney);
		markResult(resultMoney);
		return resultMoney;
	}
	
	/**
	 * 对结果进行标号
	 * @param resultMoney
	 */
	private void markResult(List<CountResultExtend> resultMoney) {
		if(ObjectUtil.empty(resultMoney)){
			return;
		}
		int i = 0 ;
		for (CountResultExtend countResultExtend : resultMoney) {
			countResultExtend.setIndex(i++);
		}
	}


	private void filterResult(List<CountResultExtend> resultMoney) {
		if(ObjectUtil.empty(resultMoney)){
			return;
		}
		for (CountResultExtend moneys : resultMoney) {
			List<CountResult> money = moneys.getCountResults();

			if(ObjectUtil.empty(money)){
				continue;
			}

			Collections.sort(money,(CountResult mm1, CountResult mm2)->{
				
				return String.valueOf(mm1.getJpCdCode()).compareTo(String.valueOf(mm2.getJpCdCode()));
			});
			
			
			Iterator<CountResult> iterator = money.iterator();
			
			CountResult temp = null;
			
			
			while(iterator.hasNext()){
				
				if(temp == null){
					temp = iterator.next();					
				}else{
					CountResult next = iterator.next();
					if(merge(temp,next))
						iterator.remove();
					else
						temp = next;
				}
				
			}
			
//			for (CountResult mm1 : money) {
//				mm1.setWay(null);
//			}
			
		}
	}

	public Boolean merge(CountResult mm1, CountResult mm2){
		if(Objects.equals(mm1.getJpCdCode(), mm2.getJpCdCode())){
			/*
			 * 累加所有费用类型总和
			 */

			Way way = mm1.getJpAmount() > 0 ? mm1.getWay() : mm2.getWay();
			mm1.setWay(way);
			if(mm2.getSAmount() != null) {
				mm1.setSAmount(mm2.getSAmount());
			}
			try {
				
				
				Double money = Double.parseDouble(mm1.getJpAmount().toString()) + 
				Double.parseDouble(mm2.getJpAmount().toString());
				// System.out.println(mm1.getJpAmount() + "   " + mm2.getJpAmount());
				// // System.out.println("金额A："+ money);
				
				Double rnm1;
				Double rnm2;
				try {
					rnm1 = Double.parseDouble(mm1.getRecordNormMoney().toString());
				} catch (Exception e) {
					rnm1 = 0D;
				} 
				try {
					 rnm2 = Double.parseDouble(mm2.getRecordNormMoney().toString());
				} catch (Exception e) {
					rnm2 = 0D;
				} 
				// System.out.println("rnm1"+ rnm1+ "rnm2"+rnm2);
				
				mm1.setRecordNormMoney( rnm1 > rnm2 ? rnm1: rnm2);
				mm2.setRecordNormMoney( rnm1 > rnm2 ? rnm1: rnm2);
				
				// System.out.println("m1:-> "+ mm1.getRecordNormMoney());
				// System.out.println("m2:-> "+ mm2.getRecordNormMoney());
				
				mm1.setJpAmount( money);
				mm2.setJpAmount( money);
				// System.out.println("金额->："+ mm1.getJpAmount());

			} catch (NumberFormatException e) {
			}
		
			
			/*
			 * 累加所有错误信息
			 */
			if(!Objects.equals(mm1.getErrorMsg(), mm2.getErrorMsg())){
				((List<AppStanderMsg>) mm1.getErrorMsg()).addAll(
						(List<AppStanderMsg>) mm2.getErrorMsg());
				mm1.setErrorMsg(mm1.getErrorMsg());
			}
			/*
			 * 累加票据必须
			 * 有一个票据必须就必须
			 * 0是不必须，1是必须
			 */
			Object billMust = mm1.getBillMust();
			if(billMust != null && Integer.parseInt(billMust.toString()) == 0){
				mm1.setBillMust( mm2.getBillMust());
			}	
			return Boolean.TRUE;
		}
		return Boolean.FALSE;
	}

	/**
	 * 计算两个时间是否在一天内
	 * @Title: isEqDate
	 * @param @param a
	 * @param @param b
	 * @param @return    设定文件
	 * @return Boolean    返回类型
	 * @throws
	 */
	public Boolean isEqDate(String a, String b){

		try {
			return  YMDTool.getDays(
						DateTool.stringToDate(a),
						DateTool.stringToDate(b)) > 1 ?
								false : true;
		} catch (Exception e) {
			return false;
		}
		
	}


	
	public Double compMode(CountResult m1, CountResult m2){
		return  Double.parseDouble(m1.getJpAmount().toString()) - Double.parseDouble(
						m2.getJpAmount().toString());
	}
	
	// 最大
	public void max(List<CountResult> find) {
		if(ObjectUtil.empty(find)){
			return;
		}
		List<CountResult> group = find;	
		CountResult  max = group.get(0);
		
		for (int i = 1; i < group.size(); i++) {
			Double compMode = compMode(group.get(i), max);
			if(compMode > 0)
				max = group.get(i);
		}
		
		group.remove(max);
		setMoney(group, 0);
	}
	
	// 最小
	public void min(List<CountResult> find) {
		if(ObjectUtil.empty(find)){
			return;
		}
		List<CountResult> group = find;	
		CountResult  max = group.get(0);
		
		for (int i = 1; i < group.size(); i++) {
			Double compMode = compMode(group.get(i), max);
			if(compMode < 0)
				max = group.get(i);
		}
		
		group.remove(max);
		setMoney(group, 0);
	}
	
	// 平均
	public void avg(List<CountResult> find) {

		if(ObjectUtil.empty(find)){
			return;
		}

		double sum = 0;
		int i = 0;
		for (CountResult m : find) {
			sum += Double.parseDouble(m.getJpAmount().toString());
			i++;
		}
		setMoney(find, 0);
		find.get(0).setJpAmount( sum/i);
	}
	
	
	// 第一个
	public void first(List<CountResult> find) {

		if(ObjectUtil.empty(find)){
			return;
		}
		find.remove(0);
		setMoney(find, 0);		
	}
	
	/**
	 * 赋值
	 * @Title: setMoney
	 * @param @param find
	 * @param @param val    设定文件
	 * @return void    返回类型
	 * @throws
	 */
	private void setMoney(List<CountResult> find, double val) {
		if(ObjectUtil.empty(find)){
			return;
		}
		for (CountResult mm : find) {
			mm.setJpAmount( val);
		}
	}


	// 最后一个
	public void last(List<CountResult> find) {
		if(ObjectUtil.empty(find)){
			return;
		}
		find.remove(find.size()-1);
		setMoney(find, 0);	
	}
}