package com.changhong.epc.count.service.count.logic.impl;

import com.changhong.epc.bean.count.MasterInfo;
import com.changhong.epc.bean.tenant.bill.IOCRBill;
import com.changhong.epc.constter.system.businesscode.BudgetBusinessMeaningCode;
import com.changhong.epc.count.service.count.model.CountResult;
import com.changhong.epc.count.service.count.model.PublicCorrencyCountResult;
import com.changhong.epc.rely.api.epc.admin.CurrencyApi;
import com.changhong.epc.rely.api.tool.CurrencyTool;
import com.iemsoft.framework.cloud.core.tools.DateTool;
import com.iemsoft.framework.cloud.core.tools.ObjectUtil;
import com.iemsoft.framework.cloud.core.tools.SpringUtil;
import lombok.extern.slf4j.Slf4j;

import java.util.List;
import java.util.Objects;
import java.util.Optional;

@Slf4j
public class PublicCorrencyCountLogic extends CountLogicImpl implements BudgetBusinessMeaningCode {

    @Override
    protected CountResult getNewCountResult() {
        return new PublicCorrencyCountResult();
    }

    @Override
    protected void initData(IOCRBill iocrBill) {
        super.initData(iocrBill);
        if(iocrBill == null){
            return;
        }
        // 给票据添加汇率
        String corrency = iocrBill.getBillKeyCode(BILL_CORRENCY);
        if(ObjectUtil.empty(corrency) || ObjectUtil.notEmpty(iocrBill.getExchangeRate())){
            return;
        }
        String applycationCorrency = getCpnVal(APPLYCATION_CURRENCY);
        // 没有（申请、报销）币种，获取本位币种
        if(ObjectUtil.empty(applycationCorrency)){
            applycationCorrency = getCpnVal(CURRENCY);
        }
        // 添加汇率
        iocrBill.setExchangeRate(
                SpringUtil.getBean(CurrencyApi.class)
                        .exchangeRate(corrency
                                , applycationCorrency
                                , Optional.ofNullable(getCpnVal(BUDGET_START_DATE)).orElse(DateTool.currentFormatDate(DateTool.DATE_PATTERN)))
        );
    }

    @Override
    protected CountResult resultMoney(double money) {
        PublicCorrencyCountResult p = (PublicCorrencyCountResult) super.resultMoney(money);
        if(iocrBill != null){
            p.getBills().add(iocrBill);
        }
        return p;
    }

    @Override
    protected double getBillMoney() {
        if(iocrBill == null){
            return 0;
        }
        if(ObjectUtil.empty(iocrBill.getExchangeRate())) {
            return super.getBillMoney();
        }else{
            return Double.valueOf(
                    CurrencyTool.valueOf(iocrBill.getExchangeRate(), Objects.toString(super.getBillMoney(), "0"))
            );
        }
    }

    @Override
    protected CountResult merge(CountResult arg1, CountResult arg2) {
        // 合并票据信息
        ((PublicCorrencyCountResult) arg1).getBills().addAll(
                ((PublicCorrencyCountResult) arg2).getBills());
        return super.merge(arg1, arg2);
    }

    private String getCpnVal(String alias) {
        return iparml.current().getUnitInfoMdJoinAll()
                .stream()
                .filter(md->Objects.equals(md.getAlias(), alias))
                .findFirst()
                .map(MasterInfo::getMeVal)
                .map(val->{
                    if(val instanceof List){
                        return ((List) val).stream().findFirst().orElse(null);
                    }else if(val instanceof String){
                        return val;
                    }else{
                        return val;
                    }
                })
                .map(Object::toString)
                .orElse(null);
    }
}
