package com.changhong.epc.count.service.count.model;

import com.alibaba.fastjson.annotation.JSONField;
import com.changhong.epc.bean.count.MasterInfo;
import com.changhong.epc.bean.tenant.bill.IOCRBill;
import com.changhong.epc.count.service.tool.MasterListTool;
import com.iemsoft.framework.cloud.core.tools.ObjectUtil;
import lombok.Data;

import java.util.*;
import java.util.stream.Collectors;

@Data
public class UnitInfo {	
	
	private UnitInfo oldUnitInfo;
	
	/**
	 * 单元id
	 */
	private Integer unitId;

	private String uuid;
	
	/**
	 * 单元主数据
	 */
	private Set<MasterInfo> unitInfo;
	
	/**
	 * 关联所有主数据
	 */
	private Set<MasterInfo> unitInfoMdJoinAll;
	
	/**
	 * 多票据信息
	 */
//	private Set<OCRBillHeaderEntity> billInfos = new HashSet<>();

	// 可能是header 也可能是body
	private List<IOCRBill> billInfo = new ArrayList<>();

	public List<IOCRBill> getBillInfo(){
		return billInfo.stream().filter(b->b != null).collect(Collectors.toList());
	}

	public void addBillInfo(List<IOCRBill> billInfo){
		this.billInfo.addAll(billInfo);
	}
//	private IOCRBill billInfo;

	@JSONField(serialize=false)
	public UnitInfo getOldUnitInfo(){
		return oldUnitInfo;
	}
	
	
	/*
	 * 缓存排序后的主数据信息
	 */
	private MasterInfo[] miArray;
	/* 二分查找主数据 */
	public MasterInfo binarySearch(MasterInfo mi){
		if(this.miArray == null){
			MasterInfo[] miSort = getUnitInfoMdJoinAll().toArray(new MasterInfo[getUnitInfoMdJoinAll().size()]);
			Arrays.sort(miSort, (MasterInfo mi1, MasterInfo mi2)->mi1.getMdCode().compareTo(mi2.getMdCode()));
			this.miArray = miSort;
		}
		int index = Arrays.binarySearch(this.miArray, mi
				, (MasterInfo mi1, MasterInfo mi2)->mi1.getMdCode().compareTo(mi2.getMdCode()));
		if(index >= 0){
			return this.miArray[index];
		}else{
			return null;
		}
	}

	public Set<MasterInfo> getUnitInfoMdJoinAll() {
		return unitInfoMdJoinAll;
	}

	public void addMdJoinAll(Boolean replace, MasterInfo... masterInfos){
		MasterListTool.add(
				replace
				, getUnitInfoMdJoinAll()
				, ml->Objects.equals("From_site", ml.getAlias()) || Objects.equals("From_site_land", ml.getAlias())
				, masterInfos);
	}

	public UnitInfo() {
		this.unitInfo = new LinkedHashSet<>();
		this.unitInfoMdJoinAll = new LinkedHashSet<>();
	}

	public void setUuid(String uuid) {
		this.uuid = uuid;
	}

	/**
	 * , Set<OCRBillHeaderEntity> billInfos
	 * @param copy
	 * @param bis
	 * @param billMaster
	 */
	public UnitInfo(UnitInfo copy,List<IOCRBill> bis, Set<MasterInfo> billMaster){
		this.unitId = copy.getUnitId();
		this.uuid = copy.getUuid();
		// 优先票据信息。
		this.unitInfo = getMasterInfo(bis);
//		this.unitInfo = new HashSet<>(bis == null ? Collections.EMPTY_SET : bis.getMasterInfo());
		this.unitInfo.addAll(copy.getUnitInfo());

		this.unitInfoMdJoinAll = new LinkedHashSet<>(copy.getUnitInfoMdJoinAll());
		this.unitInfoMdJoinAll.addAll(billMaster);
		this.billInfo = bis;
//		this.billInfos  = copy.billInfos;
		this.oldUnitInfo = copy;
	}

	public String getVal(String key){
		for(MasterInfo masterInfo: getUnitInfoMdJoinAll()){
			if(Objects.equals(masterInfo.getAlias(), key)){
				return Objects.toString(masterInfo.getMeVal(), "");
			}
		}
		return "";
	}

	public static Set<MasterInfo> getMasterInfo(List<IOCRBill> bis){
		if(ObjectUtil.empty(bis)){
			return Collections.EMPTY_SET;
		}
		return new HashSet<>(
				Arrays.asList(bis.stream().map(iocrBill -> iocrBill.getMasterInfo()).toArray(MasterInfo[]::new))
		);
	}

}
