package com.changhong.epc.count.service.format.impl;

import com.changhong.epc.bean.count.MasterInfo;
import com.changhong.epc.bean.count.format.*;
import com.changhong.epc.bean.count.format.CpnInfo.CpnMsgCode;
import com.changhong.epc.bean.count.format.CpnInfo.CpnMsgInfo;
import com.changhong.epc.bean.count.format.FormMsgInfo.GroupColor;
import com.changhong.epc.bean.count.format.FormMsgInfo.GroupData;
import com.changhong.epc.bean.tenant.bill.IOCRBill;
import com.changhong.epc.bean.tenant.norm.way.Way;
import com.changhong.epc.config.tool.ContextEnvironment;
import com.changhong.epc.constter.count.format.ConstCountMoney;
import com.changhong.epc.constter.parsing.bill.ConstBill;
import com.changhong.epc.constter.system.SystemClients;
import com.changhong.epc.constter.system.businesscode.BusinessMeaningCode;
import com.changhong.epc.constter.tenant.TenantUrlConst;
import com.changhong.epc.count.service.count.model.*;
import com.changhong.epc.count.service.format.FormatFormInfo;
import com.changhong.epc.count.service.tool.MasterListTool;
import com.iemsoft.framework.cloud.core.tools.JSONTool;
import com.iemsoft.framework.cloud.core.tools.MsgTool;
import com.iemsoft.framework.cloud.core.tools.ObjectUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

import java.math.RoundingMode;
import java.text.DecimalFormat;
import java.text.NumberFormat;
import java.util.*;
import java.util.stream.Collectors;
import java.util.stream.Stream;

/**
 * 格式化计算费用
 * @author WangYX
 *
 */
@Service("formatFormInfoCount")
@Slf4j
public class FormatFormInfoCount implements FormatFormInfo<CountParamMax, CountResultLast>,SystemClients, TenantUrlConst, ConstBill, ConstCountMoney {

	protected FormInfo formInfo;

	/**
	 * 格式计算费用参数
	 */
	@Override
	public CountParamMax formatParam(FormInfo formInfo) {
		this.formInfo = formInfo;
		CountParamMax countParam = new CountParamMax();
		if(ObjectUtil.empty(formInfo) || ObjectUtil.empty(formInfo.getData())) 
			return countParam;
		List<UnitInfo> unitInfos = new ArrayList<>(20);
		countParam.setCtId(formInfo.getCtId());
		countParam.setUnitData(unitInfos);
		countParam.setOrderCode(formInfo.getOrderCode());// findOrderCode(formInfo.getData()));
		/*
		 * 获得主控件值
		 */
		Set<MasterInfo> mastInfos = new HashSet<>(10);
		addMaster(
			formInfo
				.getData()
				.stream()
				.filter(this::isNotSubForm)
				, mastInfos);
		/*
		 * 获得行程子表单控件值
		 */
		formInfo
			.getData()
			.stream()
			.filter(this::isJourneySubForm)
			.forEach(cpnInfo->eachSubForms(cpnInfo, unitInfos, mastInfos));
		
		return countParam;
	}

	/*
	 * 验证行程子表单
	 */
	public boolean isJourneySubForm(CpnInfo cpnInfo){
		// 这里逻辑需要更新下，_journey是配置出来的。
		return isSubForm(cpnInfo) ? cpnInfo.getFieldKey().endsWith(BusinessMeaningCode.JOURNEY_SUB) : false;
	}
	
	/*
	 * 遍历子表单
	 */
	public void eachSubForms(CpnInfo subForm, List<UnitInfo> unitInfos, Set<MasterInfo> baseMaster){
		if(ObjectUtil.empty(subForm.getChild())){
			return;
		}
		List<ItemInfo> list = subForm.getChild();

		for(ItemInfo itemInfo:list){	
			Set<MasterInfo> mastInfos = new HashSet<>(20);
			mastInfos.addAll(baseMaster);
			UnitInfo unit = new UnitInfo();
			unit.setUnitId(itemInfo.getCId());
			unit.setUnitInfo(mastInfos);
			unitInfos.add(unit);
			
			// 添加主数据
			addMaster(itemInfo.getItems().stream(), mastInfos);

		}
	}
	
	/*
	 * 添加主数据
	 */
	public void addMaster(Stream<CpnInfo> stream, Set<MasterInfo> mastInfos){
		stream
			.filter(this::canAddAlias)
			.forEach(
				item-> MasterListTool.add(
						Boolean.TRUE
						, mastInfos
						, ml->Objects.equals("From_site", ml.getAlias()) || Objects.equals("From_site_land", ml.getAlias())
						, new MasterInfo(item.getMdCode(), "", item.getValue(),item.getFieldKey()))
			);
	}
	
	/**
	 * 根据主数据添加
	 * @param cpnInfo
	 * @return
	 */
	public boolean canAddMaster(CpnInfo cpnInfo){
		return ObjectUtil.notEmpty(cpnInfo.getMdCode());
	}
	
	/**
	 * 根据别名添加添加
	 * @param cpnInfo
	 * @return
	 */
	public boolean canAddAlias(CpnInfo cpnInfo){
		return true; // ObjectUtil.notEmpty(cpnInfo.getMdCode());
	}
	
	@Override
	public void formatResult(FormInfo formInfo, CountResultLast countResultLast) {
		this.formInfo = formInfo;
		CpnInfo journeyInfo = getJourneyInfo(formInfo);
		Set<GroupData> msgData = new HashSet<>();
		/**
		 * 赋值行程信息
		 */
		countResultLast.getEachMoney().forEach(
				money->sebSubFormCpnInfo(money, journeyInfo, msgData));
		/**
		 * 添加未关联票据
		 */
		addCpnInfo(
				Boolean.FALSE
				, getNoJoinBillMdCode()
				, null
				, toNumber(countResultLast.getNoJoinBillMoney())
				, 0D
				, formInfo
				, null);

		// 添加无行程票据信息
		ContextEnvironment context = new ContextEnvironment();
		if(context.travelType(formInfo.getCtId(), ()->true, ()->true, ()->false)) {
			setFormMsgInfo(formInfo
					, MsgTool.get("NO_JOIN_BILL_GROUP")
					, countResultLast
							.getNoJoinBill()
							.stream()
							.filter(iocrBill -> iocrBill.getCostType() != null)
								.map(IOCRBill::getCostType)
								.map(IOCRBill.CostType::getName)
							.filter(name->ObjectUtil.notEmpty(name))
								.map(NoJoinBillGroupData::new)
							.collect(Collectors.toSet())
			);
		}

		/**
		 * 添加错误信息
		 */
		if(ObjectUtil.empty(msgData))
			return;
		// 添加错误信息
		setFormMsgInfo(formInfo, getErrMsgName(), msgData);
	}

	public class NoJoinBillGroupData extends GroupData{

		public NoJoinBillGroupData(String data){
			setData(data);
		}

		@Override
		public String getData() {
			return super.getData() + " " + MsgTool.get("NO_JOIN_BILL");
		}
	}
	/**
	 * 添加表单错误信息
	 * @param formInfo
	 * @param groupName
	 * @param msgData
	 */
	protected void setFormMsgInfo(FormInfo formInfo, String groupName, Set<GroupData> msgData){
		if(ObjectUtil.empty(msgData)){
			return;
		}
		Set<FormMsgInfo> msgInfos = formInfo.getMsgInfo();
		FormMsgInfo msg = new FormMsgInfo(groupName);
		if(msgInfos.contains(msg))
			msgInfos.remove(msg);
		msg.setGroupColor(GroupColor.red);
		msg.setGroupData(msgData);
		msgInfos.add(msg);

	}
	
	/**
	 * 添加子表单控件信息
	 * @param money
	 * @param subForm
	 * @param msgData
	 */
	public void sebSubFormCpnInfo(CountResultExtend money, CpnInfo subForm, Set<GroupData> msgData){
		// 未赋值到控件中的费用信息
		List<CountResult> noCpn = new ArrayList<>(5);
		money.getCountResults().forEach(
			result->{
				/*
				 * 添加行程控件值及提示
				 */
				if(!addCpnInfo(
						Way.DEBG == result.getWay()
						, result.getJpCdCode()
						, result.getJpAmount()
						, result.getBillMoney()
						, result.getJpAmount()
						, Optional.ofNullable(subForm).map(CpnInfo::getChild).map(c->c.get(Optional.ofNullable(money).map(CountResultExtend::getIndex).orElse(0))).orElse(null)
						, result)
				) {

					noCpn.add(result);
				}
				/*
				 * 添加不在标准内错误信息
				 */
				if(ObjectUtil.notEmpty(result.getErrorMsg()))
					result.getErrorMsg().stream().forEach(err->msgData.add(new GroupData(err.getErrMsg())));
			});
		/*
		 * 赋值其它费用
		 */
		if(ObjectUtil.empty(noCpn))
			return;
		double amount = 0, billMoney = 0;
		for(CountResult count: noCpn){
			amount 		+= count.getJpAmount() == null 	? 0 : count.getJpAmount();
			billMoney 	+= count.getBillMoney() == null ? 0 : count.getBillMoney();
		}
		log.debug("其他费用标准金额：{}", JSONTool.toJson(noCpn));
		addCpnInfo(
				Boolean.FALSE
				, getOtherMdCode()
				, amount
				, billMoney + money.getOtherMoney()
				, amount
				, subForm.getChild().get(money.getIndex())
				, null);
	}
	
	/**
	 * 其它费用主数据
	 * @return
	 */
	public String getOtherMdCode() {
		return OTHER_COST;
	}
	
	/**
	 * 未关联行程票据主数据
	 * @return
	 */
	public String getNoJoinBillMdCode(){
		return E_OTHER_AMOUNT;
	}
	
	/**
	 * 显示的错误标题
	 * @return
	 */
	public String getErrMsgName(){
		return MsgTool.get(NO_STANDARD_GROUP);
	}

	/**
	 * 添加控件信息
	 * @param mdCode
	 * @param jpAmount
	 * @param billMoney
	 * @param itemInfo
	 * @return
	 */
	public boolean addCpnInfo(
			boolean replace
			, String mdCode
			, Double jpAmount
			, Double billMoney
			, Double cpnVal
			, CpnInfoCommons itemInfo
			, CountResult countResult){
		if(itemInfo == null){
			return false;
		}
		TreeSet<CpnMsgInfo> msgInfos = new TreeSet<>();
		if(jpAmount != null)
			msgInfos.add(getNomrCpnMsgInfo(jpAmount, true, countResult)); // 标准
		if(billMoney != null && billMoney > 0)
			msgInfos.add(getBillCpnMsgInfo(billMoney, false, countResult)); // 票据
		boolean add = false;
		try {
			return add = itemInfo.addCpnVal(mdCode, cpnVal, replace)
					? itemInfo.addCpnMsg(mdCode, CpnMsgCode.header, msgInfos)
					: false;
		}finally {
			log.debug("添加{}控件值：{} 票据值：{} 消息：{} 添加：{}"
					, mdCode
					, jpAmount
					, billMoney
					, JSONTool.toJson(msgInfos)
					, add ? "成功" : "无此控件");
		}
	}

	public static Double toNumber(Double d) {
		NumberFormat nf = new DecimalFormat("0.00");

		// 保留两位小数
//        nf.setMaximumFractionDigits(2);

		// 如果不需要四舍五入，可以使用RoundingMode.DOWN
		nf.setRoundingMode(RoundingMode.UP);

		return new Double(nf.format(d));
	}

	protected CpnMsgInfo getNomrCpnMsgInfo(Double jpAmount, boolean validate, CountResult countResult){
		return new CpnMsgInfo(MsgTool.get(NOMR_TITLE_NAME), toNumber(jpAmount), validate);
	}

	protected CpnMsgInfo getBillCpnMsgInfo(Double billMoney, boolean validate, CountResult countResult){
		return new CpnMsgInfo(MsgTool.get(BILL_TITLE_NAME), billMoney, validate);
	}

}
